@@ -483,6 +483,7 @@ template<> struct assert_intrinsic_wchar_t<unsigned short> {};
 // BOOST_NO_CXX11_ALIGNAS
 #if (BOOST_INTEL_CXX_VERSION >= 1500) && (!defined(BOOST_INTEL_GCC_VERSION) || (BOOST_INTEL_GCC_VERSION >= 40800)) && (!defined(_MSC_VER) || (_MSC_FULL_VER >= 190021730))
 #  undef BOOST_NO_CXX11_ALIGNAS
+#  undef BOOST_NO_CXX11_ALIGNOF
 #endif
 
 // BOOST_NO_CXX11_TRAILING_RESULT_TYPES
@@ -501,8 +502,15 @@ template<> struct assert_intrinsic_wchar_t<unsigned short> {};
 #endif
 
 // BOOST_NO_CXX11_FINAL
+// BOOST_NO_CXX11_OVERRIDE
 #if (BOOST_INTEL_CXX_VERSION >= 1400) && (!defined(BOOST_INTEL_GCC_VERSION) || (BOOST_INTEL_GCC_VERSION >= 40700)) && (!defined(_MSC_VER) || (_MSC_VER >= 1700))
 #  undef BOOST_NO_CXX11_FINAL
+#  undef BOOST_NO_CXX11_OVERRIDE
+#endif
+
+// BOOST_NO_CXX11_UNRESTRICTED_UNION
+#if (BOOST_INTEL_CXX_VERSION >= 1400) && (!defined(BOOST_INTEL_GCC_VERSION) || (BOOST_INTEL_GCC_VERSION >= 50100)) && (!defined(_MSC_VER))
+#  undef BOOST_NO_CXX11_UNRESTRICTED_UNION
 #endif
 
 #endif // defined(BOOST_INTEL_STDCXX0X)
