@@ -9,22 +9,20 @@
 
 #include <boost/math/tools/config.hpp>
 #include <boost/math/tools/mp.hpp>
-#include <limits>
-#include <type_traits>
-#include <cmath>
-#include <cstdint>
-#include <cstddef>
+#include <boost/math/tools/numeric_limits.hpp>
+#include <boost/math/tools/type_traits.hpp>
+#include <boost/math/tools/cstdint.hpp>
 
-namespace mp = boost::math::tools::meta_programming;
+namespace boost{ namespace math{
 
-namespace boost{ namespace math{ 
+namespace mp = tools::meta_programming;
 
 namespace tools{
 
 template <class T>
-constexpr int digits(BOOST_MATH_EXPLICIT_TEMPLATE_TYPE(T)) noexcept;
+BOOST_MATH_GPU_ENABLED constexpr int digits(BOOST_MATH_EXPLICIT_TEMPLATE_TYPE(T)) noexcept;
 template <class T>
-constexpr T epsilon(BOOST_MATH_EXPLICIT_TEMPLATE_TYPE(T)) noexcept(std::is_floating_point<T>::value);
+BOOST_MATH_GPU_ENABLED constexpr T epsilon(BOOST_MATH_EXPLICIT_TEMPLATE_TYPE(T)) noexcept(boost::math::is_floating_point<T>::value);
 
 }
 
@@ -33,9 +31,36 @@ namespace policies{
 //
 // Define macros for our default policies, if they're not defined already:
 //
+
+
+//
+// Generic support for GPUs
+//
+#ifdef BOOST_MATH_HAS_GPU_SUPPORT
+#  ifndef BOOST_MATH_OVERFLOW_ERROR_POLICY
+#    define BOOST_MATH_OVERFLOW_ERROR_POLICY ignore_error
+#  endif
+#  ifndef BOOST_MATH_PROMOTE_DOUBLE_POLICY
+#     define BOOST_MATH_PROMOTE_DOUBLE_POLICY false
+#  endif
+#  ifndef BOOST_MATH_DOMAIN_ERROR_POLICY
+#    define BOOST_MATH_DOMAIN_ERROR_POLICY ignore_error
+#  endif
+#  ifndef BOOST_MATH_POLE_ERROR_POLICY
+#     define BOOST_MATH_POLE_ERROR_POLICY ignore_error
+#  endif
+#  ifndef BOOST_MATH_EVALUATION_ERROR_POLICY
+#     define BOOST_MATH_EVALUATION_ERROR_POLICY ignore_error
+#  endif
+#  ifndef BOOST_MATH_ROUNDING_ERROR_POLICY
+#     define BOOST_MATH_ROUNDING_ERROR_POLICY ignore_error
+#  endif
+#endif
+
+//
 // Special cases for exceptions disabled first:
 //
-#ifdef BOOST_NO_EXCEPTIONS
+#ifdef BOOST_MATH_NO_EXCEPTIONS
 #  ifndef BOOST_MATH_DOMAIN_ERROR_POLICY
 #    define BOOST_MATH_DOMAIN_ERROR_POLICY errno_on_error
 #  endif
@@ -107,22 +132,22 @@ namespace policies{
 
 #define BOOST_MATH_META_INT(Type, name, Default)                                                \
    template <Type N = Default>                                                                  \
-   class name : public std::integral_constant<int, N>{};                                        \
+   class name : public boost::math::integral_constant<int, N>{};                                \
                                                                                                 \
    namespace detail{                                                                            \
    template <Type N>                                                                            \
-   char test_is_valid_arg(const name<N>* = nullptr);                                            \
-   char test_is_default_arg(const name<Default>* = nullptr);                                    \
+   BOOST_MATH_GPU_ENABLED char test_is_valid_arg(const name<N>* = nullptr);                     \
+   BOOST_MATH_GPU_ENABLED char test_is_default_arg(const name<Default>* = nullptr);             \
                                                                                                 \
    template <typename T>                                                                        \
    class is_##name##_imp                                                                        \
    {                                                                                            \
    private:                                                                                     \
       template <Type N>                                                                         \
-      static char test(const name<N>* = nullptr);                                               \
-      static double test(...);                                                                  \
+      BOOST_MATH_GPU_ENABLED static char test(const name<N>* = nullptr);                        \
+      BOOST_MATH_GPU_ENABLED static double test(...);                                           \
    public:                                                                                      \
-      static constexpr bool value = sizeof(test(static_cast<T*>(0))) == sizeof(char);           \
+      static constexpr bool value = sizeof(test(static_cast<T*>(nullptr))) == sizeof(char);     \
    };                                                                                           \
    }                                                                                            \
                                                                                                 \
@@ -131,27 +156,27 @@ namespace policies{
    {                                                                                            \
    public:                                                                                      \
       static constexpr bool value = boost::math::policies::detail::is_##name##_imp<T>::value;   \
-      using type = std::integral_constant<bool, value>;                                         \
+      using type = boost::math::integral_constant<bool, value>;                                 \
    };
 
 #define BOOST_MATH_META_BOOL(name, Default)                                                     \
    template <bool N = Default>                                                                  \
-   class name : public std::integral_constant<bool, N>{};                                       \
+   class name : public boost::math::integral_constant<bool, N>{};                               \
                                                                                                 \
    namespace detail{                                                                            \
    template <bool N>                                                                            \
-   char test_is_valid_arg(const name<N>* = nullptr);                                            \
-   char test_is_default_arg(const name<Default>* = nullptr);                                    \
+   BOOST_MATH_GPU_ENABLED char test_is_valid_arg(const name<N>* = nullptr);                     \
+   BOOST_MATH_GPU_ENABLED char test_is_default_arg(const name<Default>* = nullptr);             \
                                                                                                 \
    template <typename T>                                                                        \
    class is_##name##_imp                                                                        \
    {                                                                                            \
    private:                                                                                     \
       template <bool N>                                                                         \
-      static char test(const name<N>* = nullptr);                                               \
-      static double test(...);                                                                  \
+      BOOST_MATH_GPU_ENABLED static char test(const name<N>* = nullptr);                        \
+      BOOST_MATH_GPU_ENABLED static double test(...);                                           \
    public:                                                                                      \
-      static constexpr bool value = sizeof(test(static_cast<T*>(0))) == sizeof(char);           \
+      static constexpr bool value = sizeof(test(static_cast<T*>(nullptr))) == sizeof(char);     \
    };                                                                                           \
    }                                                                                            \
                                                                                                 \
@@ -160,7 +185,7 @@ namespace policies{
    {                                                                                            \
    public:                                                                                      \
       static constexpr bool value = boost::math::policies::detail::is_##name##_imp<T>::value;   \
-      using type = std::integral_constant<bool, value>;                                         \
+      using type = boost::math::integral_constant<bool, value>;                                 \
    };
 
 //
@@ -232,37 +257,37 @@ struct precision
    //
    // Now work out the precision:
    //
-   using digits2_type = typename std::conditional<
+   using digits2_type = typename boost::math::conditional<
       (Digits10::value == 0),
       digits2<0>,
       digits2<((Digits10::value + 1) * 1000L) / 301L>
    >::type;
 public:
 #ifdef BOOST_BORLANDC
-   using type = typename std::conditional<
+   using type = typename boost::math::conditional<
       (Digits2::value > ::boost::math::policies::detail::precision<Digits10,Digits2>::digits2_type::value),
       Digits2, digits2_type>::type;
 #else
-   using type = typename std::conditional<
+   using type = typename boost::math::conditional<
       (Digits2::value > digits2_type::value),
       Digits2, digits2_type>::type;
 #endif
 };
 
-double test_is_valid_arg(...);
-double test_is_default_arg(...);
-char test_is_valid_arg(const default_policy*);
-char test_is_default_arg(const default_policy*);
+BOOST_MATH_GPU_ENABLED double test_is_valid_arg(...);
+BOOST_MATH_GPU_ENABLED double test_is_default_arg(...);
+BOOST_MATH_GPU_ENABLED char test_is_valid_arg(const default_policy*);
+BOOST_MATH_GPU_ENABLED char test_is_default_arg(const default_policy*);
 
 template <typename T>
-class is_valid_policy_imp 
+class is_valid_policy_imp
 {
 public:
-   static constexpr bool value = sizeof(boost::math::policies::detail::test_is_valid_arg(static_cast<T*>(0))) == sizeof(char);
+   static constexpr bool value = sizeof(boost::math::policies::detail::test_is_valid_arg(static_cast<T*>(nullptr))) == sizeof(char);
 };
 
-template <typename T> 
-class is_valid_policy 
+template <typename T>
+class is_valid_policy
 {
 public:
    static constexpr bool value = boost::math::policies::detail::is_valid_policy_imp<T>::value;
@@ -272,7 +297,7 @@ template <typename T>
 class is_default_policy_imp
 {
 public:
-   static constexpr bool value = sizeof(boost::math::policies::detail::test_is_default_arg(static_cast<T*>(0))) == sizeof(char);
+   static constexpr bool value = sizeof(boost::math::policies::detail::test_is_default_arg(static_cast<T*>(nullptr))) == sizeof(char);
 };
 
 template <typename T>
@@ -280,7 +305,7 @@ class is_default_policy
 {
 public:
    static constexpr bool value = boost::math::policies::detail::is_default_policy_imp<T>::value;
-   using type = std::integral_constant<bool, value>;
+   using type = boost::math::integral_constant<bool, value>;
 
    template <typename U>
    struct apply
@@ -289,7 +314,7 @@ public:
    };
 };
 
-template <class Seq, class T, std::size_t N>
+template <class Seq, class T, boost::math::size_t N>
 struct append_N
 {
    using type = typename append_N<mp::mp_push_back<Seq, T>, T, N-1>::type;
@@ -342,8 +367,8 @@ typedef default_args<BOOST_MATH_PROMOTE_FLOAT_POLICY, BOOST_MATH_PROMOTE_DOUBLE_
 // Now define the policy type with enough arguments to handle all
 // the policies:
 //
-template <typename A1  = default_policy, 
-          typename A2  = default_policy, 
+template <typename A1  = default_policy,
+          typename A2  = default_policy,
           typename A3  = default_policy,
           typename A4  = default_policy,
           typename A5  = default_policy,
@@ -378,7 +403,7 @@ private:
    // Typelist of the arguments:
    //
    using arg_list = mp::mp_list<A1,A2,A3,A4,A5,A6,A7,A8,A9,A10,A11,A12,A13>;
-   static constexpr std::size_t arg_list_size = mp::mp_size<arg_list>::value;
+   static constexpr boost::math::size_t arg_list_size = mp::mp_size<arg_list>::value;
 
    template<typename A, typename B, bool b>
    struct pick_arg
@@ -417,7 +442,7 @@ public:
    using evaluation_error_type = typename arg_type<mp::mp_quote_trait<is_evaluation_error>, evaluation_error<>>::type;
    using rounding_error_type = typename arg_type<mp::mp_quote_trait<is_rounding_error>, rounding_error<>>::type;
    using indeterminate_result_error_type = typename arg_type<mp::mp_quote_trait<is_indeterminate_result_error>, indeterminate_result_error<>>::type;
-   
+
    // Precision:
    using precision_type = typename detail::precision<digits10_type, bits_precision_type>::type;
 
@@ -427,7 +452,7 @@ public:
 
    // Discrete quantiles:
    using discrete_quantile_type = typename arg_type<mp::mp_quote_trait<is_discrete_quantile>, discrete_quantile<>>::type;
-   
+
    // Mathematically undefined properties:
    using assert_undefined_type = typename arg_type<mp::mp_quote_trait<is_assert_undefined>, assert_undefined<>>::type;
 
@@ -491,9 +516,9 @@ public:
    using max_root_iterations_type = max_root_iterations<>;
 };
 
-template <typename Policy, 
-          typename A1  = default_policy, 
-          typename A2  = default_policy, 
+template <typename Policy,
+          typename A1  = default_policy,
+          typename A2  = default_policy,
           typename A3  = default_policy,
           typename A4  = default_policy,
           typename A5  = default_policy,
@@ -509,7 +534,7 @@ class normalise
 {
 private:
    using arg_list = mp::mp_list<A1,A2,A3,A4,A5,A6,A7,A8,A9,A10,A11,A12,A13>;
-   static constexpr std::size_t arg_list_size = mp::mp_size<arg_list>::value;
+   static constexpr boost::math::size_t arg_list_size = mp::mp_size<arg_list>::value;
 
    template<typename A, typename B, bool b>
    struct pick_arg
@@ -583,7 +608,7 @@ private:
    // Remove all the policies that are the same as the default:
    using fn = mp::mp_quote_trait<detail::is_default_policy>;
    using reduced_list = mp::mp_remove_if_q<result_list, fn>;
-   
+
    // Pad out the list with defaults:
    using result_type = typename detail::append_N<reduced_list, default_policy, (14UL - mp::mp_size<reduced_list>::value)>::type;
 
@@ -607,9 +632,9 @@ public:
 
 // Full specialisation to speed up compilation of the common case:
 template <>
-struct normalise<policy<>, 
-          promote_float<false>, 
-          promote_double<false>, 
+struct normalise<policy<>,
+          promote_float<false>,
+          promote_double<false>,
           discrete_quantile<>,
           assert_undefined<>,
           default_policy,
@@ -640,81 +665,81 @@ struct normalise<policy<detail::forwarding_arg1, detail::forwarding_arg2>,
    using type = policy<detail::forwarding_arg1, detail::forwarding_arg2>;
 };
 
-inline constexpr policy<> make_policy() noexcept
-{ return policy<>(); }
+BOOST_MATH_GPU_ENABLED constexpr policy<> make_policy() noexcept
+{ return {}; }
 
 template <class A1>
-inline constexpr typename normalise<policy<>, A1>::type make_policy(const A1&) noexcept
-{ 
+BOOST_MATH_GPU_ENABLED constexpr typename normalise<policy<>, A1>::type make_policy(const A1&) noexcept
+{
    typedef typename normalise<policy<>, A1>::type result_type;
-   return result_type(); 
+   return result_type();
 }
 
 template <class A1, class A2>
-inline constexpr typename normalise<policy<>, A1, A2>::type make_policy(const A1&, const A2&) noexcept
-{ 
+BOOST_MATH_GPU_ENABLED constexpr typename normalise<policy<>, A1, A2>::type make_policy(const A1&, const A2&) noexcept
+{
    typedef typename normalise<policy<>, A1, A2>::type result_type;
-   return result_type(); 
+   return result_type();
 }
 
 template <class A1, class A2, class A3>
-inline constexpr typename normalise<policy<>, A1, A2, A3>::type make_policy(const A1&, const A2&, const A3&) noexcept
-{ 
+BOOST_MATH_GPU_ENABLED constexpr typename normalise<policy<>, A1, A2, A3>::type make_policy(const A1&, const A2&, const A3&) noexcept
+{
    typedef typename normalise<policy<>, A1, A2, A3>::type result_type;
-   return result_type(); 
+   return result_type();
 }
 
 template <class A1, class A2, class A3, class A4>
-inline constexpr typename normalise<policy<>, A1, A2, A3, A4>::type make_policy(const A1&, const A2&, const A3&, const A4&) noexcept
-{ 
+BOOST_MATH_GPU_ENABLED constexpr typename normalise<policy<>, A1, A2, A3, A4>::type make_policy(const A1&, const A2&, const A3&, const A4&) noexcept
+{
    typedef typename normalise<policy<>, A1, A2, A3, A4>::type result_type;
-   return result_type(); 
+   return result_type();
 }
 
 template <class A1, class A2, class A3, class A4, class A5>
-inline constexpr typename normalise<policy<>, A1, A2, A3, A4, A5>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&) noexcept
-{ 
+BOOST_MATH_GPU_ENABLED constexpr typename normalise<policy<>, A1, A2, A3, A4, A5>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&) noexcept
+{
    typedef typename normalise<policy<>, A1, A2, A3, A4, A5>::type result_type;
-   return result_type(); 
+   return result_type();
 }
 
 template <class A1, class A2, class A3, class A4, class A5, class A6>
-inline constexpr typename normalise<policy<>, A1, A2, A3, A4, A5, A6>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&, const A6&) noexcept
-{ 
+BOOST_MATH_GPU_ENABLED constexpr typename normalise<policy<>, A1, A2, A3, A4, A5, A6>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&, const A6&) noexcept
+{
    typedef typename normalise<policy<>, A1, A2, A3, A4, A5, A6>::type result_type;
-   return result_type(); 
+   return result_type();
 }
 
 template <class A1, class A2, class A3, class A4, class A5, class A6, class A7>
-inline constexpr typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&, const A6&, const A7&) noexcept
-{ 
+BOOST_MATH_GPU_ENABLED constexpr typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&, const A6&, const A7&) noexcept
+{
    typedef typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7>::type result_type;
-   return result_type(); 
+   return result_type();
 }
 
 template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
-inline constexpr typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7, A8>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&, const A6&, const A7&, const A8&) noexcept
-{ 
+BOOST_MATH_GPU_ENABLED constexpr typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7, A8>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&, const A6&, const A7&, const A8&) noexcept
+{
    typedef typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7, A8>::type result_type;
-   return result_type(); 
+   return result_type();
 }
 
 template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9>
-inline constexpr typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7, A8, A9>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&, const A6&, const A7&, const A8&, const A9&) noexcept
-{ 
+BOOST_MATH_GPU_ENABLED constexpr typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7, A8, A9>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&, const A6&, const A7&, const A8&, const A9&) noexcept
+{
    typedef typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7, A8, A9>::type result_type;
-   return result_type(); 
+   return result_type();
 }
 
 template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9, class A10>
-inline constexpr typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&, const A6&, const A7&, const A8&, const A9&, const A10&) noexcept
-{ 
+BOOST_MATH_GPU_ENABLED constexpr typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&, const A6&, const A7&, const A8&, const A9&, const A10&) noexcept
+{
    typedef typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10>::type result_type;
-   return result_type(); 
+   return result_type();
 }
 
 template <class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8, class A9, class A10, class A11>
-inline constexpr typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10, A11>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&, const A6&, const A7&, const A8&, const A9&, const A10&, const A11&) noexcept
+BOOST_MATH_GPU_ENABLED constexpr typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10, A11>::type make_policy(const A1&, const A2&, const A3&, const A4&, const A5&, const A6&, const A7&, const A8&, const A9&, const A10&, const A11&) noexcept
 {
    typedef typename normalise<policy<>, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10, A11>::type result_type;
    return result_type();
@@ -732,47 +757,47 @@ struct evaluation
 template <class Policy>
 struct evaluation<float, Policy>
 {
-   using type = typename std::conditional<Policy::promote_float_type::value, double, float>::type;
+   using type = typename boost::math::conditional<Policy::promote_float_type::value, double, float>::type;
 };
 
 template <class Policy>
 struct evaluation<double, Policy>
 {
-   using type = typename std::conditional<Policy::promote_double_type::value, long double, double>::type;
+   using type = typename boost::math::conditional<Policy::promote_double_type::value, long double, double>::type;
 };
 
 template <class Real, class Policy>
 struct precision
 {
-   static_assert((std::numeric_limits<Real>::radix == 2) || ((std::numeric_limits<Real>::is_specialized == 0) || (std::numeric_limits<Real>::digits == 0)),
-   "(std::numeric_limits<Real>::radix == 2) || ((std::numeric_limits<Real>::is_specialized == 0) || (std::numeric_limits<Real>::digits == 0))");
+   static_assert((boost::math::numeric_limits<Real>::radix == 2) || ((boost::math::numeric_limits<Real>::is_specialized == 0) || (boost::math::numeric_limits<Real>::digits == 0)),
+   "(boost::math::numeric_limits<Real>::radix == 2) || ((boost::math::numeric_limits<Real>::is_specialized == 0) || (boost::math::numeric_limits<Real>::digits == 0))");
 #ifndef BOOST_BORLANDC
    using precision_type = typename Policy::precision_type;
-   using type = typename std::conditional<
-      ((std::numeric_limits<Real>::is_specialized == 0) || (std::numeric_limits<Real>::digits == 0)),
+   using type = typename boost::math::conditional<
+      ((boost::math::numeric_limits<Real>::is_specialized == 0) || (boost::math::numeric_limits<Real>::digits == 0)),
       // Possibly unknown precision:
       precision_type,
-      typename std::conditional<
-         ((std::numeric_limits<Real>::digits <= precision_type::value) 
+      typename boost::math::conditional<
+         ((boost::math::numeric_limits<Real>::digits <= precision_type::value)
          || (Policy::precision_type::value <= 0)),
          // Default case, full precision for RealType:
-         digits2< std::numeric_limits<Real>::digits>,
+         digits2< boost::math::numeric_limits<Real>::digits>,
          // User customised precision:
          precision_type
       >::type
    >::type;
 #else
    using precision_type = typename Policy::precision_type;
-   using digits_t = std::integral_constant<int, std::numeric_limits<Real>::digits>;
-   using spec_t = std::integral_constant<bool, std::numeric_limits<Real>::is_specialized>;
-   using type = typename std::conditional<
-      (spec_t::value == true std::true_type || digits_t::value == 0),
+   using digits_t = boost::math::integral_constant<int, boost::math::numeric_limits<Real>::digits>;
+   using spec_t = boost::math::integral_constant<bool, boost::math::numeric_limits<Real>::is_specialized>;
+   using type = typename boost::math::conditional<
+      (spec_t::value == true boost::math::true_type || digits_t::value == 0),
       // Possibly unknown precision:
       precision_type,
-      typename std::conditional<
+      typename boost::math::conditional<
          (digits_t::value <= precision_type::value || precision_type::value <= 0),
          // Default case, full precision for RealType:
-         digits2< std::numeric_limits<Real>::digits>,
+         digits2< boost::math::numeric_limits<Real>::digits>,
          // User customised precision:
          precision_type
       >::type
@@ -785,7 +810,7 @@ struct precision
 template <class Policy>
 struct precision<BOOST_MATH_FLOAT128_TYPE, Policy>
 {
-   typedef std::integral_constant<int, 113> type;
+   typedef boost::math::integral_constant<int, 113> type;
 };
 
 #endif
@@ -793,15 +818,15 @@ struct precision<BOOST_MATH_FLOAT128_TYPE, Policy>
 namespace detail{
 
 template <class T, class Policy>
-inline constexpr int digits_imp(std::true_type const&) noexcept
+BOOST_MATH_GPU_ENABLED constexpr int digits_imp(boost::math::true_type const&) noexcept
 {
-   static_assert( std::numeric_limits<T>::is_specialized, "std::numeric_limits<T>::is_specialized");
+   static_assert( boost::math::numeric_limits<T>::is_specialized, "boost::math::numeric_limits<T>::is_specialized");
    typedef typename boost::math::policies::precision<T, Policy>::type p_t;
    return p_t::value;
 }
 
 template <class T, class Policy>
-inline constexpr int digits_imp(std::false_type const&) noexcept
+BOOST_MATH_GPU_ENABLED constexpr int digits_imp(boost::math::false_type const&) noexcept
 {
    return tools::digits<T>();
 }
@@ -809,26 +834,26 @@ inline constexpr int digits_imp(std::false_type const&) noexcept
 } // namespace detail
 
 template <class T, class Policy>
-inline constexpr int digits(BOOST_MATH_EXPLICIT_TEMPLATE_TYPE(T)) noexcept
+BOOST_MATH_GPU_ENABLED constexpr int digits(BOOST_MATH_EXPLICIT_TEMPLATE_TYPE(T)) noexcept
 {
-   typedef std::integral_constant<bool, std::numeric_limits<T>::is_specialized > tag_type;
+   typedef boost::math::integral_constant<bool, boost::math::numeric_limits<T>::is_specialized > tag_type;
    return detail::digits_imp<T, Policy>(tag_type());
 }
 template <class T, class Policy>
-inline constexpr int digits_base10(BOOST_MATH_EXPLICIT_TEMPLATE_TYPE(T)) noexcept
+BOOST_MATH_GPU_ENABLED constexpr int digits_base10(BOOST_MATH_EXPLICIT_TEMPLATE_TYPE(T)) noexcept
 {
    return boost::math::policies::digits<T, Policy>() * 301 / 1000L;
 }
 
 template <class Policy>
-inline constexpr unsigned long get_max_series_iterations() noexcept
+BOOST_MATH_GPU_ENABLED constexpr unsigned long get_max_series_iterations() noexcept
 {
    typedef typename Policy::max_series_iterations_type iter_type;
    return iter_type::value;
 }
 
 template <class Policy>
-inline constexpr unsigned long get_max_root_iterations() noexcept
+BOOST_MATH_GPU_ENABLED constexpr unsigned long get_max_root_iterations() noexcept
 {
    typedef typename Policy::max_root_iterations_type iter_type;
    return iter_type::value;
@@ -839,51 +864,51 @@ namespace detail{
 template <class T, class Digits, class Small, class Default>
 struct series_factor_calc
 {
-   static T get() noexcept(std::is_floating_point<T>::value)
+   BOOST_MATH_GPU_ENABLED static T get() noexcept(boost::math::is_floating_point<T>::value)
    {
       return ldexp(T(1.0), 1 - Digits::value);
    }
 };
 
 template <class T, class Digits>
-struct series_factor_calc<T, Digits, std::true_type, std::true_type>
+struct series_factor_calc<T, Digits, boost::math::true_type, boost::math::true_type>
 {
-   static constexpr T get() noexcept(std::is_floating_point<T>::value)
+   BOOST_MATH_GPU_ENABLED static constexpr T get() noexcept(boost::math::is_floating_point<T>::value)
    {
       return boost::math::tools::epsilon<T>();
    }
 };
 template <class T, class Digits>
-struct series_factor_calc<T, Digits, std::true_type, std::false_type>
+struct series_factor_calc<T, Digits, boost::math::true_type, boost::math::false_type>
 {
-   static constexpr T get() noexcept(std::is_floating_point<T>::value)
+   BOOST_MATH_GPU_ENABLED static constexpr T get() noexcept(boost::math::is_floating_point<T>::value)
    {
-      return 1 / static_cast<T>(static_cast<std::uintmax_t>(1u) << (Digits::value - 1));
+      return 1 / static_cast<T>(static_cast<boost::math::uintmax_t>(1u) << (Digits::value - 1));
    }
 };
 template <class T, class Digits>
-struct series_factor_calc<T, Digits, std::false_type, std::true_type>
+struct series_factor_calc<T, Digits, boost::math::false_type, boost::math::true_type>
 {
-   static constexpr T get() noexcept(std::is_floating_point<T>::value)
+   BOOST_MATH_GPU_ENABLED static constexpr T get() noexcept(boost::math::is_floating_point<T>::value)
    {
       return boost::math::tools::epsilon<T>();
    }
 };
 
 template <class T, class Policy>
-inline constexpr T get_epsilon_imp(std::true_type const&) noexcept(std::is_floating_point<T>::value)
+BOOST_MATH_GPU_ENABLED constexpr T get_epsilon_imp(boost::math::true_type const&) noexcept(boost::math::is_floating_point<T>::value)
 {
-   static_assert(std::numeric_limits<T>::is_specialized, "std::numeric_limits<T>::is_specialized");
-   static_assert(std::numeric_limits<T>::radix == 2, "std::numeric_limits<T>::radix == 2");
+   static_assert(boost::math::numeric_limits<T>::is_specialized, "boost::math::numeric_limits<T>::is_specialized");
+   static_assert(boost::math::numeric_limits<T>::radix == 2, "boost::math::numeric_limits<T>::radix == 2");
 
    typedef typename boost::math::policies::precision<T, Policy>::type p_t;
-   typedef std::integral_constant<bool, p_t::value <= std::numeric_limits<std::uintmax_t>::digits> is_small_int;
-   typedef std::integral_constant<bool, p_t::value >= std::numeric_limits<T>::digits> is_default_value;
+   typedef boost::math::integral_constant<bool, p_t::value <= boost::math::numeric_limits<boost::math::uintmax_t>::digits> is_small_int;
+   typedef boost::math::integral_constant<bool, p_t::value >= boost::math::numeric_limits<T>::digits> is_default_value;
    return series_factor_calc<T, p_t, is_small_int, is_default_value>::get();
 }
 
 template <class T, class Policy>
-inline constexpr T get_epsilon_imp(std::false_type const&) noexcept(std::is_floating_point<T>::value)
+BOOST_MATH_GPU_ENABLED constexpr T get_epsilon_imp(boost::math::false_type const&) noexcept(boost::math::is_floating_point<T>::value)
 {
    return tools::epsilon<T>();
 }
@@ -891,16 +916,16 @@ inline constexpr T get_epsilon_imp(std::false_type const&) noexcept(std::is_floa
 } // namespace detail
 
 template <class T, class Policy>
-inline constexpr T get_epsilon(BOOST_MATH_EXPLICIT_TEMPLATE_TYPE(T)) noexcept(std::is_floating_point<T>::value)
+BOOST_MATH_GPU_ENABLED constexpr T get_epsilon(BOOST_MATH_EXPLICIT_TEMPLATE_TYPE(T)) noexcept(boost::math::is_floating_point<T>::value)
 {
-   typedef std::integral_constant<bool, (std::numeric_limits<T>::is_specialized && (std::numeric_limits<T>::radix == 2)) > tag_type;
+   typedef boost::math::integral_constant<bool, (boost::math::numeric_limits<T>::is_specialized && (boost::math::numeric_limits<T>::radix == 2)) > tag_type;
    return detail::get_epsilon_imp<T, Policy>(tag_type());
 }
 
 namespace detail{
 
-template <class A1, 
-          class A2, 
+template <class A1,
+          class A2,
           class A3,
           class A4,
           class A5,
@@ -910,14 +935,14 @@ template <class A1,
           class A9,
           class A10,
           class A11>
-char test_is_policy(const policy<A1,A2,A3,A4,A5,A6,A7,A8,A9,A10,A11>*);
-double test_is_policy(...);
+BOOST_MATH_GPU_ENABLED char test_is_policy(const policy<A1,A2,A3,A4,A5,A6,A7,A8,A9,A10,A11>*);
+BOOST_MATH_GPU_ENABLED double test_is_policy(...);
 
 template <typename P>
 class is_policy_imp
 {
 public:
-   static constexpr bool value = (sizeof(::boost::math::policies::detail::test_is_policy(static_cast<P*>(0))) == sizeof(char));
+   static constexpr bool value = (sizeof(::boost::math::policies::detail::test_is_policy(static_cast<P*>(nullptr))) == sizeof(char));
 };
 
 }
@@ -927,7 +952,7 @@ class is_policy
 {
 public:
    static constexpr bool value = boost::math::policies::detail::is_policy_imp<P>::value;
-   using type = std::integral_constant<bool, value>;
+   using type = boost::math::integral_constant<bool, value>;
 };
 
 //
@@ -937,20 +962,20 @@ template <class Policy>
 struct constructor_error_check
 {
    using domain_error_type = typename Policy::domain_error_type;
-   using type = typename std::conditional<
+   using type = typename boost::math::conditional<
       (domain_error_type::value == throw_on_error) || (domain_error_type::value == user_error) || (domain_error_type::value == errno_on_error),
-      std::true_type,
-      std::false_type>::type;
+      boost::math::true_type,
+      boost::math::false_type>::type;
 };
 
 template <class Policy>
 struct method_error_check
 {
    using domain_error_type = typename Policy::domain_error_type;
-   using type = typename std::conditional<
-      (domain_error_type::value == throw_on_error) && (domain_error_type::value != user_error),
-      std::false_type,
-      std::true_type>::type;
+   using type = typename boost::math::conditional<
+      (domain_error_type::value == throw_on_error),
+      boost::math::false_type,
+      boost::math::true_type>::type;
 };
 //
 // Does the Policy ever throw on error?
@@ -967,7 +992,7 @@ struct is_noexcept_error_policy
    typedef typename Policy::rounding_error_type             t7;
    typedef typename Policy::indeterminate_result_error_type t8;
 
-   static constexpr bool value = 
+   static constexpr bool value =
       ((t1::value != throw_on_error) && (t1::value != user_error)
       && (t2::value != throw_on_error) && (t2::value != user_error)
       && (t3::value != throw_on_error) && (t3::value != user_error)
